/* celloutG.c -- cellular automata/motif interface for RTcmix */
#include <Xm/Xm.h>
#include <Xm/BulletinB.h>
#include <Xm/DrawingA.h>

#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/signal.h>
#include <sys/time.h>

#include "/musr/cmix/H/RTsockfuncs.h"

int theSock; /* RTcmix socket */

Widget topLevel, bboard;
Widget theDraw;

XtAppContext app;

GC gc;
Colormap cmap;
XColor bground,fground,unused;

#define NCELLS 16
#define NGENS 40
#define CWIDTH 20

int drawwidth,drawheight;

/* music stuff */
int cells[2][NCELLS];
int cur,next;
float beat;
int curgen;
float pitches[10] = { 6.10, 7.05, 7.07, 7.10, 8.00, 8.02, 8.03, 8.05, 8.07, 8.10 };

main(argc,argv)
int argc;
char *argv[];
{
	extern int RTsock(char*,int);
	struct sockdata ssend;
	struct timeval tv;
	struct itimerval itv;
	void gocells();
	float brrand();
	void srrand();
	float fval;
	int i;

/* set up the RTcmix connection */
	system("STRUM -i -n &");
	sleep(1);
	theSock = RTsock("localhost", 0);

	strcpy(ssend.name, "rtsetparams");
	ssend.data.p[0] = 44100;
	ssend.data.p[1] = 1;
	ssend.n_args = 2;
	write(theSock, (char *)&ssend, sizeof(struct sockdata));


/* and now the graphics stuff */
	topLevel=XtVaAppInitialize(&app,"CellautoG",
		NULL,0,&argc,argv,NULL,NULL);

	/* set up the main window */
	drawwidth = NCELLS * CWIDTH;
	drawheight = NGENS * CWIDTH;
	bboard = XtVaCreateManagedWidget("bboard",
		xmBulletinBoardWidgetClass, topLevel,
		XmNwidth, drawwidth, XmNheight, drawheight,
		XmNmarginWidth, 0, XmNmarginHeight, 0,NULL);

	/* set up the colors */
	cmap = DefaultColormapOfScreen(XtScreen(bboard));
	XAllocNamedColor(XtDisplay(bboard),cmap,"deepskyblue",&bground,&unused);
	XAllocNamedColor(XtDisplay(bboard),cmap,"deeppink3",&fground,&unused);
	gc = XCreateGC(XtDisplay(bboard),
		RootWindowOfScreen(XtScreen(bboard)), 0, NULL);

	/* set up the drawing area in the main window */
	theDraw = XtVaCreateManagedWidget("theDraw",
		xmDrawingAreaWidgetClass, bboard,
		XmNwidth, drawwidth, XmNheight, drawheight,
		XmNbackground, bground.pixel, NULL);

/* set up the music params */
	beat = 0.1;
	curgen = 0;

	/* initialize the first row */
	cur = 0;
	next = 1;
	for (i = 0; i < NCELLS; i++) {
		fval = brrand();
		if (fval > 0.5) cells[cur][i] = 1;
		else cells[cur][i] = 0;
	}

/* set up the timer */
	tv.tv_sec = (int)(beat * (float)NCELLS);
	tv.tv_usec = (int)(((beat*(float)NCELLS)-(float)(tv.tv_sec))*1000000.0);
	itv.it_interval = tv;
	itv.it_value = tv;
	setitimer(ITIMER_REAL, &itv, NULL);
	signal(SIGALRM, gocells);


	XtManageChild(bboard);
	XtManageChild(theDraw);
	XtRealizeWidget(topLevel);

	XtAppMainLoop(app);
}



void gocells()
{
	struct sockdata ssend;
	int j;
	float brrand();
	int right,left,combine;

	/* go through the cells */
	XSetForeground(XtDisplay(theDraw), gc, fground.pixel);
	for (j = 0; j < NCELLS; j++) {

		if (cells[cur][j] == 1) {
			/* draw it! */
			XFillRectangle(XtDisplay(theDraw),XtWindow(theDraw),gc,
				j*CWIDTH,curgen*CWIDTH,CWIDTH,CWIDTH);

			/* sound it! */
			strcpy(ssend.name, "START");
			ssend.data.p[0] = (float)j * beat;
			ssend.data.p[1] = 0.5;
/*			ssend.data.p[2] = pitches[(int)(brrand() * 10.0)];
*/
			ssend.data.p[2] = 9.00;
			ssend.data.p[3] = 0.5;
			ssend.data.p[4] = 0.3;
			ssend.data.p[5] = 7000;
			ssend.data.p[6] = 0;
			ssend.n_args = 7;
			write(theSock, (char *)&ssend, sizeof(struct sockdata));
		}
		else {
			/* sound it! */
			strcpy(ssend.name, "START");
			ssend.data.p[0] = (float)j * beat;
			ssend.data.p[1] = 0.5;
			ssend.data.p[2] = 7.00;
			ssend.data.p[3] = 0.5;
			ssend.data.p[4] = 0.3;
			ssend.data.p[5] = 5000;
			ssend.data.p[6] = 3;
			ssend.n_args = 7;
			write(theSock, (char *)&ssend, sizeof(struct sockdata));
		}

		/* now calculate the next row element */
		/* find the neighbors */
		if (j == 0) left = 0;
		else left = cells[cur][j-1];
		if (j == NCELLS-1) right = 1;
		else right = cells[cur][j+1];

		combine = left + right;
		if (combine == 0) {
			cells[next][j] = 0;
		}
		if (combine == 1) {
			cells[next][j] = 1;
		}
		if (combine == 2) {
			cells[next][j] = 0;
		}
	}
	/* now set for the next round */
	cur = next;
	if (++next > 1) next = 0;
	if (++curgen > NGENS) {
		curgen = 0;
		XSetForeground(XtDisplay(theDraw), gc, bground.pixel);
		XFillRectangle(XtDisplay(theDraw),XtWindow(theDraw),gc,
			0,0,NCELLS*CWIDTH,NGENS*CWIDTH);
	}

	XFlush(XtDisplay(theDraw));
}



long randx = 1;

void srrand(x)
unsigned x;
{
	randx = x;
}

float brrand()
{
	int i = ((randx = randx*1103515245 + 12345)>>16) & 077777;
	return((float)i/32768.0);
}



